@@ -37,6 +37,7 @@ from .hashing import hash
 from .func_inspect import get_func_code, get_func_name, filter_args
 from .logger import Logger, format_time
 from . import numpy_pickle
+from .disk import rm_subdirs
 
 FIRST_LINE_TEXT = "# first line:"
 
@@ -75,7 +76,7 @@ class JobLibCollisionWarning(UserWarning):
 
 
 ################################################################################
-# class `Memory`
+# class `MemorizedFunc`
 ################################################################################
 class MemorizedFunc(Logger):
     """ Callable object decorating a function for caching its return value 
@@ -107,7 +108,7 @@ class MemorizedFunc(Logger):
     #-------------------------------------------------------------------------
    
     def __init__(self, func, cachedir, ignore=None, save_npy=True, 
-                             mmap_mode=None, verbose=1):
+                             mmap_mode=None, verbose=1, timestamp=None):
         """
             Parameters
             ----------
@@ -127,7 +128,10 @@ class MemorizedFunc(Logger):
                 cache was created.
             verbose: int, optional
                 Verbosity flag, controls the debug messages that are issued 
-                as functions are revaluated.
+                as functions are revaluated. The higher, the more verbose
+            timestamp: float, optional
+                The reference time from which times in tracing messages
+                are reported.
         """
         Logger.__init__(self)
         self._verbose = verbose
@@ -135,6 +139,9 @@ class MemorizedFunc(Logger):
         self.func = func
         self.save_npy = save_npy
         self.mmap_mode = mmap_mode
+        if timestamp is None:
+            timestamp = time.time()
+        self.timestamp = timestamp
         if ignore is None:
             ignore = []
         self.ignore = ignore
@@ -156,25 +163,41 @@ class MemorizedFunc(Logger):
     def __call__(self, *args, **kwargs):
         # Compare the function code with the previous to see if the
         # function code has changed
-        output_dir = self.get_output_dir(*args, **kwargs)
+        output_dir, _ = self.get_output_dir(*args, **kwargs)
         # FIXME: The statements below should be try/excepted
         if not (self._check_previous_func_code(stacklevel=3) and 
                                  os.path.exists(output_dir)):
             return self.call(*args, **kwargs)
         else:
             try:
-                return self.load_output(output_dir)
-            except Exception, e:
+                t0 = time.time()
+                out = self.load_output(output_dir)
+                if self._verbose > 4:
+                    t = time.time() - t0
+                    _, name = get_func_name(self.func)
+                    msg = '%s cache loaded - %s' % (name, format_time(t))
+                    print max(0, (80 - len(msg)))*'_' + msg
+                return out
+            except Exception:
                 # XXX: Should use an exception logger
                 self.warn(
                 'Exception while loading results for '
                 '(args=%s, kwargs=%s)\n %s' %
                     (args, kwargs, traceback.format_exc())
                     )
-                      
-                shutil.rmtree(output_dir)
+
+                shutil.rmtree(output_dir, ignore_errors=True)
                 return self.call(*args, **kwargs)
 
+
+    def __reduce__(self):
+        """ We don't store the timestamp when pickling, to avoid the hash
+            depending from it.
+            In addition, when unpickling, we run the __init__
+        """
+        return (self.__class__, (self.func, self.cachedir, self.ignore, 
+                self.save_npy, self.mmap_mode, self._verbose))
+
     #-------------------------------------------------------------------------
     # Private interface
     #-------------------------------------------------------------------------
@@ -209,7 +232,7 @@ class MemorizedFunc(Logger):
                              coerce_mmap=coerce_mmap)
         output_dir = os.path.join(self._get_func_dir(self.func),
                                     argument_hash)
-        return output_dir
+        return output_dir, argument_hash
         
 
     def _write_func_code(self, filename, func_code, first_line):
@@ -231,11 +254,14 @@ class MemorizedFunc(Logger):
         func_dir = self._get_func_dir()
         func_code_file = os.path.join(func_dir, 'func_code.py')
 
-        if not os.path.exists(func_code_file): 
-            self._write_func_code(func_code_file, func_code, first_line)
-            return False
-        old_func_code, old_first_line = \
-                        extract_first_line(file(func_code_file).read())
+        try:
+            if not os.path.exists(func_code_file): 
+                raise IOError
+            old_func_code, old_first_line = \
+                            extract_first_line(file(func_code_file).read())
+        except IOError:
+                self._write_func_code(func_code_file, func_code, first_line)
+                return False
         if old_func_code == func_code:
             return True
 
@@ -244,7 +270,8 @@ class MemorizedFunc(Logger):
         # changed?
 
         if old_first_line == first_line == -1:
-            _, func_name = get_func_name(self.func, resolv_alias=False)
+            _, func_name = get_func_name(self.func, resolv_alias=False,
+                                         win_characters=False)
             if not first_line == -1:
                 func_description = '%s (%s:%i)' % (func_name, 
                                                 source_file, first_line)
@@ -287,8 +314,12 @@ class MemorizedFunc(Logger):
         if self._verbose and warn:
             self.warn("Clearing cache %s" % func_dir)
         if os.path.exists(func_dir):
-            shutil.rmtree(func_dir)
-        os.makedirs(func_dir)
+            shutil.rmtree(func_dir, ignore_errors=True)
+        try:
+            os.makedirs(func_dir)
+        except OSError:
+            """ Directory exists: it has been created by another process
+            in the mean time. """
         func_code, _, first_line = get_func_code(self.func)
         func_code_file = os.path.join(func_dir, 'func_code.py')
         self._write_func_code(func_code_file, func_code, first_line)
@@ -298,17 +329,17 @@ class MemorizedFunc(Logger):
         """ Force the execution of the function with the given arguments and 
             persist the output values.
         """
+        start_time = time.time()
         if self._verbose:
             print self.format_call(*args, **kwargs)
-            start_time = time.time()
-        output_dir = self.get_output_dir(*args, **kwargs)
+        output_dir, argument_hash = self.get_output_dir(*args, **kwargs)
         output = self.func(*args, **kwargs)
         self._persist_output(output, output_dir)
-        self._persist_input(output_dir, *args, **kwargs)
+        input_repr = self._persist_input(output_dir, *args, **kwargs)
+        duration = time.time() - start_time
         if self._verbose:
             _, name = get_func_name(self.func)
-            msg = '%s - %s' % (name, 
-                               format_time(time.time() - start_time))
+            msg = '%s - %s' % (name, format_time(duration))
             print max(0, (80 - len(msg)))*'_' + msg
         return output
 
@@ -356,16 +387,19 @@ class MemorizedFunc(Logger):
     def _persist_output(self, output, dir):
         """ Persist the given output tuple in the directory.
         """
-        if not os.path.exists(dir):
-            os.makedirs(dir)
-        filename = os.path.join(dir, 'output.pkl')
+        try:
+            if not os.path.exists(dir):
+                os.makedirs(dir)
+            filename = os.path.join(dir, 'output.pkl')
 
-        if 'numpy' in sys.modules and self.save_npy:
-            numpy_pickle.dump(output, filename) 
-        else:
-            output_file = file(filename, 'w')
-            pickle.dump(output, output_file, protocol=2)
-            output_file.close()
+            if 'numpy' in sys.modules and self.save_npy:
+                numpy_pickle.dump(output, filename) 
+            else:
+                output_file = file(filename, 'w')
+                pickle.dump(output, output_file, protocol=2)
+                output_file.close()
+        except OSError:
+            " Race condition in the creation of the directory "
 
 
     def _persist_input(self, output_dir, *args, **kwargs):
@@ -374,17 +408,32 @@ class MemorizedFunc(Logger):
         """
         argument_dict = filter_args(self.func, self.ignore,
                                     *args, **kwargs)
+
+        input_repr = dict((k, repr(v)) for k, v in argument_dict.iteritems())
         if json is not None:
-            json.dump(
-                dict((k, repr(v)) 
-                    for k, v in argument_dict.iteritems()),
-                file(os.path.join(output_dir, 'input_args.json'), 'w'),
-                )
+            # This can fail do to race-conditions with multiple
+            # concurrent joblibs removing the file or the directory
+            try:
+                if not os.path.exists(output_dir):
+                    os.makedirs(output_dir)
+                json.dump(
+                    input_repr,
+                    file(os.path.join(output_dir, 'input_args.json'), 'w'),
+                    )
+            except:
+                pass
+        return input_repr
 
     def load_output(self, output_dir):
         """ Read the results of a previous calculation from the directory
             it was cached in.
         """
+        if self._verbose > 1:
+            t = time.time() - self.timestamp
+            print '[Memory]% 16s: Loading %s...' % (
+                                    format_time(t),
+                                    self.format_signature(self.func)[0]
+                                    )
         filename = os.path.join(output_dir, 'output.pkl')
         if self.save_npy:
             return numpy_pickle.load(filename, 
@@ -448,17 +497,36 @@ class Memory(Logger):
         # XXX: Bad explaination of the None value of cachedir
         Logger.__init__(self)
         self._verbose = verbose
-        self.cachedir = cachedir
         self.save_npy = save_npy
         self.mmap_mode = mmap_mode
-        if cachedir is not None and not os.path.exists(self.cachedir):
-            os.makedirs(self.cachedir)
+        self.timestamp = time.time()
+        if cachedir is None:
+            self.cachedir = None
+        else:
+            self.cachedir = os.path.join(cachedir, 'joblib')
+            if not os.path.exists(self.cachedir):
+                os.makedirs(self.cachedir)
 
 
-    def cache(self, func=None, ignore=None):
+    def cache(self, func=None, ignore=None, verbose=None,
+                        mmap_mode=False):
         """ Decorates the given function func to only compute its return
             value for input arguments not cached on disk.
 
+            Parameters
+            ----------
+            func: callable, optional
+                The function to be decorated
+            ignore: list of strings
+                A list of arguments name to ignore in the hashing
+            verbose: integer, optional
+                The verbosity mode of the function. By default that
+                of the memory object is used.
+            mmap_mode: {None, 'r+', 'r', 'w+', 'c'}, optional
+                The memmapping mode used when loading from cache
+                numpy arrays. See numpy.load for the meaning of the
+                arguments. By default that of the memory object is used.
+
             Returns
             -------
             decorated_func: MemorizedFunc object
@@ -473,11 +541,18 @@ class Memory(Logger):
             return functools.partial(self.cache, ignore=ignore)
         if self.cachedir is None:
             return func
+        if verbose is None:
+            verbose = self._verbose
+        if mmap_mode is False:
+            mmap_mode = self.mmap_mode
+        if isinstance(func, MemorizedFunc):
+            func = func.func
         return MemorizedFunc(func, cachedir=self.cachedir,
                                    save_npy=self.save_npy,
-                                   mmap_mode=self.mmap_mode,
+                                   mmap_mode=mmap_mode,
                                    ignore=ignore,
-                                   verbose=self._verbose)
+                                   verbose=verbose,
+                                   timestamp=self.timestamp)
 
 
     def clear(self, warn=True):
@@ -485,8 +560,7 @@ class Memory(Logger):
         """
         if warn:
             self.warn('Flushing completely the cache')
-        shutil.rmtree(self.cachedir)
-        os.makedirs(self.cachedir)
+        rm_subdirs(self.cachedir)
 
 
     def eval(self, func, *args, **kwargs):
