@@ -1,28 +1,22 @@
 """
-Test the format_stack module.
+Unit tests for the stack formatting utilities 
 """
 
 # Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org> 
 # Copyright (c) 2010 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
-from ..format_stack import safe_repr
-
 import nose
 
+from ..format_stack import safe_repr
+
+
 ################################################################################
-class Nasty(object):
+
+class Vicious(object):
     def __repr__(self):
         raise ValueError
 
-    __str__ = __repr__
-
-################################################################################
-# Test parallel
 
 def test_safe_repr():
-    """ Smoke test safe_repr on a nasty class.
-    """
-    nasty = Nasty()
-    safe_repr(nasty)
-
+    safe_repr(Vicious())
