@@ -8,8 +8,9 @@ Test the func_inspect module.
 
 import nose
 import tempfile
+import functools
 
-from ..func_inspect import filter_args, get_func_name
+from ..func_inspect import filter_args, get_func_name, get_func_code
 from ..memory import Memory
 
 ################################################################################
@@ -38,6 +39,9 @@ def i(x=1):
 def j(x, y, **kwargs):
     pass
 
+def k(*args, **kwargs):
+    pass
+
 class Klass(object):
 
     def f(self, x):
@@ -53,6 +57,7 @@ def test_filter_args():
     yield nose.tools.assert_equal, filter_args(f, ['y'], 0, y=1), {'x': 0}
     yield nose.tools.assert_equal, filter_args(f, ['x', 'y'], 0), {}
     yield nose.tools.assert_equal, filter_args(f, [], 0, y=1), {'x':0, 'y':1}
+    yield nose.tools.assert_equal, filter_args(f, ['y'], x=2, y=1), {'x':2}
 
     yield nose.tools.assert_equal, filter_args(i, [], 2), {'x': 2}
     yield nose.tools.assert_equal, filter_args(f2, [], x=1), {'x': 1}
@@ -74,9 +79,28 @@ def test_filter_varargs():
     yield nose.tools.assert_equal, filter_args(h, ['*'], 1, 2, 25, ee=2), \
                             {'x': 1, 'y': 2, '**':{'ee':2}}
 
-def test_tmp():
+
+def test_filter_kwargs():
+    nose.tools.assert_equal(filter_args(k, [], 1, 2, ee=2), 
+                            {'*': [1, 2], '**':{'ee':2}})
+    nose.tools.assert_equal(filter_args(k, [], 3, 4), 
+                            {'*': [3, 4], '**':{}})
+
+
+def test_filter_args_2():
     nose.tools.assert_equal(filter_args(j, [], 1, 2, ee=2), 
                             {'x': 1, 'y': 2, '**':{'ee':2}})
+    
+    nose.tools.assert_raises(ValueError, filter_args, f, 'a', None)
+    # Check that we capture an undefined argument
+    nose.tools.assert_raises(ValueError, filter_args, f, ['a'], None)
+    ff = functools.partial(f, 1)
+    # filter_args has to special-case partial
+    nose.tools.assert_equal(filter_args(ff, [], 1), 
+                            {'*': [1], '**':{}})
+    nose.tools.assert_equal(filter_args(ff, ['y'], 1), 
+                            {'*': [1], '**':{}})
+
 
 def test_func_name():
     yield nose.tools.assert_equal, 'f', get_func_name(f)[1]
@@ -84,6 +108,25 @@ def test_func_name():
     yield nose.tools.assert_equal, 'g', get_func_name(g)[1]
 
 
+def test_func_inspect_errors():
+    """ Check that func_inspect is robust and will work on weird objects
+    """
+    nose.tools.assert_equal(get_func_name('a'.lower)[-1], 'lower')
+    nose.tools.assert_equal(get_func_code('a'.lower)[1:], (None, -1))
+    ff = lambda x: x
+    nose.tools.assert_equal(get_func_name(ff, win_characters=False)[-1], 
+                                                            '<lambda>')
+    nose.tools.assert_equal(get_func_code(ff)[1], 
+                                    __file__.replace('.pyc', '.py'))
+    # Simulate a function defined in __main__
+    ff.__module__ = '__main__'
+    nose.tools.assert_equal(get_func_name(ff, win_characters=False)[-1], 
+                                                            '<lambda>')
+    nose.tools.assert_equal(get_func_code(ff)[1], 
+                                    __file__.replace('.pyc', '.py'))
+
+
+
 def test_bound_methods():
     """ Make sure that calling the same method on two different instances
         of the same class does resolv to different signatures.
